@@ -1125,27 +1125,23 @@ get_item_marker_str (HTMLClueFlow *flow, gboolean ascii_only)
 static void
 draw_gt_line (HTMLObject *cur, HTMLPainter *p, gint offset, gint x, gint y)
 {
-	gint cy, w, a, d, line_offset = 0;
+	gint cy, w, a, d;
 
 	/* FIXME: cache items and glyphs? */
-	html_painter_calc_text_size (p, HTML_BLOCK_CITE, 
-				     strlen (HTML_BLOCK_CITE), NULL, NULL, NULL, 0, &line_offset,
-				     GTK_HTML_FONT_STYLE_SIZE_3, NULL,
+	html_painter_calc_text_size (p, HTML_BLOCK_CITE, strlen (HTML_BLOCK_CITE),
 				     &w, &a, &d);
 
 	cy = offset;
 	while (cy + a <= cur->ascent) {
 		/* FIXME: cache items and glyphs? */
-		html_painter_draw_text (p, x, y + cur->y - cy,
-					HTML_BLOCK_CITE, 1, NULL, NULL, NULL, 0, 0);
+		html_painter_draw_text (p, x, y + cur->y - cy, HTML_BLOCK_CITE, 1);
 		cy += a + d;
 	}
 
 	cy = - offset + a + d;
 	while (cy + d <= cur->descent) {
 		/* FIXME: cache items and glyphs? */
-		html_painter_draw_text (p, x, y + cur->y + cy,
-					HTML_BLOCK_CITE, 1, NULL, NULL, NULL, 0, 0);
+		html_painter_draw_text (p, x, y + cur->y + cy, HTML_BLOCK_CITE, 1);
 		cy += a + d;
 	}
 }
@@ -1246,19 +1242,18 @@ draw_item (HTMLObject *self, HTMLPainter *painter, gint x, gint y, gint width, g
 
 	marker = get_item_marker_str (flow, HTML_IS_PLAIN_PAINTER (painter));
 	if (marker) {
-		gint width, len, line_offset = 0, asc, dsc;
+		gint width, len, asc, dsc;
 		
 		len   = g_utf8_strlen (marker, -1);
-		/* FIXME: cache items and glyphs? */
-		html_painter_calc_text_size (painter, marker, len, NULL, NULL, NULL, 0, &line_offset,
-					     html_clueflow_get_default_font_style (flow), NULL, &width, &asc, &dsc);
-		width += html_painter_get_space_width (painter, html_clueflow_get_default_font_style (flow), NULL);
 		html_painter_set_font_style (painter, html_clueflow_get_default_font_style (flow));
 		html_painter_set_font_face  (painter, NULL);
 		/* FIXME: cache items and glyphs? */
+		html_painter_calc_text_size (painter, marker, len, &width, &asc, &dsc);
+		width += html_painter_get_space_width (painter, html_clueflow_get_default_font_style (flow), NULL);
+		/* FIXME: cache items and glyphs? */
 		html_painter_draw_text (painter, self->x + first->x - width + tx,
 					self->y - self->ascent + first->y + ty,
-					marker, len, NULL, NULL, NULL, 0, 0);
+					marker, len);
 	}
 	g_free (marker);
 }
@@ -1854,7 +1849,7 @@ save_plain (HTMLObject *self,
 			PangoAttrList *attrs = pango_attr_list_new ();
 			gint bytes = html_engine_save_buffer_peek_text_bytes (buffer_state), slen = g_utf8_strlen (s, -1), i, clen, n_items;
 			GList *items_list, *cur;
-			PangoContext *pc = gtk_widget_get_pango_context (GTK_WIDGET (state->engine->widget));
+			PangoContext *pc = state->engine->painter->pango_context;
 			PangoLogAttr *lattrs;
 			PangoItem **items;
 			gint len, width, skip;
@@ -2308,6 +2303,16 @@ html_clueflow_type_init (void)
 	html_clueflow_class_init (&html_clueflow_class, HTML_TYPE_CLUEFLOW, sizeof (HTMLClueFlow));
 }
 
+static HTMLDirection
+html_clueflow_real_get_direction (HTMLObject *o)
+{
+	if (HTML_CLUEFLOW (o)->dir == HTML_DIRECTION_DERIVED && o->parent) {
+		return html_object_get_direction (o->parent);
+	}
+
+	return HTML_CLUEFLOW (o)->dir;
+}
+
 void
 html_clueflow_class_init (HTMLClueFlowClass *klass,
 			  HTMLType type,
@@ -2343,6 +2348,7 @@ html_clueflow_class_init (HTMLClueFlowClass *klass,
 	object_class->get_recursive_length = get_recursive_length;
 	object_class->get_clear = get_clear;
 	object_class->set_painter = set_painter;
+	object_class->get_direction = html_clueflow_real_get_direction;
 
 	klass->get_default_font_style = get_default_font_style;
 
@@ -2366,6 +2372,7 @@ html_clueflow_init (HTMLClueFlow *clueflow, HTMLClueFlowClass *klass,
 
 	clue->valign = HTML_VALIGN_BOTTOM;
 	clue->halign = HTML_HALIGN_NONE;
+	clueflow->dir = HTML_DIRECTION_DERIVED;
 
 	clueflow->style = style;
 	clueflow->levels = levels; 
@@ -2603,13 +2610,30 @@ html_clueflow_get_halignment (HTMLClueFlow *flow)
 	g_return_val_if_fail (flow != NULL, HTML_HALIGN_NONE);
 
 	if (HTML_CLUE (flow)->halign == HTML_HALIGN_NONE) {
+		HTMLHAlignType halign;
+
 		if (HTML_OBJECT (flow)->parent && HTML_IS_TABLE_CELL (HTML_OBJECT (flow)->parent))
-			return HTML_CLUE (HTML_OBJECT (flow)->parent)->halign == HTML_HALIGN_NONE
-				? HTML_TABLE_CELL (HTML_OBJECT (flow)->parent)->heading ? HTML_HALIGN_CENTER : HTML_HALIGN_LEFT
+			halign = HTML_CLUE (HTML_OBJECT (flow)->parent)->halign == HTML_HALIGN_NONE
+				? HTML_TABLE_CELL (HTML_OBJECT (flow)->parent)->heading ? HTML_HALIGN_CENTER : HTML_HALIGN_NONE
 				: HTML_CLUE (HTML_OBJECT (flow)->parent)->halign;
 		else
-			return HTML_CLUE (HTML_OBJECT (flow)->parent)->halign == HTML_HALIGN_NONE
-				? HTML_HALIGN_LEFT : HTML_CLUE (HTML_OBJECT (flow)->parent)->halign;
+			halign = HTML_CLUE (HTML_OBJECT (flow)->parent)->halign;
+
+		if (halign == HTML_HALIGN_NONE) {
+			switch (html_object_get_direction (HTML_OBJECT (flow))) {
+			case HTML_DIRECTION_LTR:
+				halign = HTML_HALIGN_LEFT;
+				break;
+			case HTML_DIRECTION_RTL:
+				halign = HTML_HALIGN_RIGHT;
+				break;
+			default:
+				break;
+			}
+		}
+
+		return halign;
+
 	} else
 		return HTML_CLUE (flow)->halign;
 }
